@@ -168,20 +168,30 @@ input_complete(int arg UNUSED, int key UNUSED)
 static int
 input_help(int arg, int key UNUSED)
 {
-  int i = 0;
+  int i, in_string, in_bracket;
 
   if (arg != 1)
     return rl_insert(arg, '?');
-  while (i < rl_point)
+
+  in_string = in_bracket = 0;
+  for (i = 0; i < rl_point; i++)
     {
-      if (rl_line_buffer[i++] == '"')
-	do
-	  {
-	    if (i >= rl_point)		/* `?' inside quoted string -> insert */
-	      return rl_insert(1, '?');
-	  }
-        while (rl_line_buffer[i++] != '"');
+   
+      if (rl_line_buffer[i] == '"')
+	in_string = ! in_string;
+      else if (! in_string)
+        {
+	  if (rl_line_buffer[i] == '[')
+	    in_bracket++;
+	  else if (rl_line_buffer[i] == ']')
+	    in_bracket--;
+        }
     }
+
+  /* `?' inside string or path -> insert */
+  if (in_string || in_bracket)
+    return rl_insert(1, '?');
+
   rl_begin_undo_group();		/* HACK: We want to display `?' at point position */
   rl_insert_text("?");
   rl_redisplay();
@@ -201,8 +211,9 @@ input_init(void)
   rl_add_defun("bird-help", input_help, '?');
   rl_callback_handler_install("bird> ", got_line);
   input_initialized = 1;
-  if (fcntl(0, F_SETFL, O_NONBLOCK) < 0)
-    die("fcntl: %m");
+//  readline library does strange things when stdin is nonblocking.
+//  if (fcntl(0, F_SETFL, O_NONBLOCK) < 0)
+//    die("fcntl: %m");
 }
 
 static void
@@ -298,11 +309,18 @@ server_read(void)
   int c;
   byte *start, *p;
 
+ redo:
   c = read(server_fd, server_read_pos, server_read_buf + sizeof(server_read_buf) - server_read_pos);
   if (!c)
     die("Connection closed by server.");
   if (c < 0)
-    die("Server read error: %m");
+    {
+      if (errno == EINTR)
+	goto redo;
+      else
+	die("Server read error: %m");
+    }
+
   start = server_read_buf;
   p = server_read_pos;
   server_read_pos += c;
@@ -331,6 +349,7 @@ static fd_set select_fds;
 static void
 select_loop(int mode)
 {
+  int rv;
   server_reply = -1;
   while (mode || server_reply < 0)
     {
@@ -338,7 +357,16 @@ select_loop(int mode)
       FD_SET(server_fd, &select_fds);
       if (mode)
 	FD_SET(0, &select_fds);
-      select(server_fd+1, &select_fds, NULL, NULL, NULL);
+
+      rv = select(server_fd+1, &select_fds, NULL, NULL, NULL);
+      if (rv < 0)
+	{
+	  if (errno == EINTR)
+	    continue;
+	  else
+	    die("select: %m");
+	}
+
       if (FD_ISSET(server_fd, &select_fds))
 	{
 	  server_read();
@@ -351,6 +379,30 @@ select_loop(int mode)
   input_reveal();
 }
 
+static void
+wait_for_write(int fd)
+{
+  while (1)
+    {
+      int rv;
+      fd_set set;
+      FD_ZERO(&set);
+      FD_SET(fd, &set);
+
+      rv = select(fd+1, NULL, &set, NULL, NULL);
+      if (rv < 0)
+	{
+	  if (errno == EINTR)
+	    continue;
+	  else
+	    die("select: %m");
+	}
+
+      if (FD_ISSET(server_fd, &set))
+	return;
+    }
+}
+
 static void
 server_send(char *cmd)
 {
@@ -362,19 +414,13 @@ server_send(char *cmd)
   while (l)
     {
       int cnt = write(server_fd, z, l);
+
       if (cnt < 0)
 	{
-	  if (errno == -EAGAIN)
-	    {
-	      fd_set set;
-	      FD_ZERO(&set);
-	      do
-		{
-		  FD_SET(server_fd, &set);
-		  select(server_fd+1, NULL, &set, NULL, NULL);
-		}
-	      while (!FD_ISSET(server_fd, &set));
-	    }
+	  if (errno == EAGAIN)
+	    wait_for_write(server_fd);
+	  else if (errno == EINTR)
+	    continue;
 	  else
 	    die("Server write error: %m");
 	}
