@@ -82,10 +82,10 @@ pm_format(struct f_path_mask *p, byte *buf, unsigned int size)
 	  return;
 	}
 
-      if (p->any)
-	buf += bsprintf(buf, "? ");
+      if (p->kind == PM_ASN)
+	buf += bsprintf(buf, " %u", p->val);
       else
-	buf += bsprintf(buf, "%u ", p->val);
+	buf += bsprintf(buf, (p->kind == PM_ASTERISK) ? " *" : " ?");
 
       p = p->next;
     }
@@ -137,12 +137,33 @@ val_compare(struct f_val v1, struct f_val v2)
     return 0;
   case T_PATH_MASK:
     return pm_path_compare(v1.val.path_mask, v2.val.path_mask);
+  case T_STRING:
+    return strcmp(v1.val.s, v2.val.s);
   default:
     debug( "Compare of unkown entities: %x\n", v1.type );
     return CMP_ERROR;
   }
 }
 
+
+void
+f_prefix_get_bounds(struct f_prefix *px, int *l, int *h)
+{
+  *l = *h = px->len & LEN_MASK;
+
+  if (px->len & LEN_MINUS)
+    *l = 0;
+
+  else if (px->len & LEN_PLUS)
+    *h = MAX_PREFIX_LENGTH;
+
+  else if (px->len & LEN_RANGE)
+    {
+      *l = 0xff & (px->len >> 16);
+      *h = 0xff & (px->len >> 8);
+    }
+}
+
 /*
  * val_simple_in_range - check if @v1 ~ @v2 for everything except sets
  */ 
@@ -153,26 +174,28 @@ val_simple_in_range(struct f_val v1, struct f_val v2)
     return as_path_match(v1.val.ad, v2.val.path_mask);
   if ((v1.type == T_PAIR) && (v2.type == T_CLIST))
     return int_set_contains(v2.val.ad, v1.val.i);
+  if ((v1.type == T_STRING) && (v2.type == T_STRING))
+    return patmatch(v2.val.s, v1.val.s);
 
   if ((v1.type == T_IP) && (v2.type == T_PREFIX))
     return !(ipa_compare(ipa_and(v2.val.px.ip, ipa_mkmask(v2.val.px.len)), ipa_and(v1.val.px.ip, ipa_mkmask(v2.val.px.len))));
 
   if ((v1.type == T_PREFIX) && (v2.type == T_PREFIX)) {
-    ip_addr mask;
+
     if (v1.val.px.len & (LEN_PLUS | LEN_MINUS | LEN_RANGE))
       return CMP_ERROR;
-    mask = ipa_mkmask( v2.val.px.len & LEN_MASK );
+
+    int p1 = v1.val.px.len & LEN_MASK;
+    int p2 = v2.val.px.len & LEN_MASK;
+    ip_addr mask = ipa_mkmask(MIN(p1, p2));
+
     if (ipa_compare(ipa_and(v2.val.px.ip, mask), ipa_and(v1.val.px.ip, mask)))
       return 0;
 
-    if ((v2.val.px.len & LEN_MINUS) && (v1.val.px.len <= (v2.val.px.len & LEN_MASK)))
-      return 0;
-    if ((v2.val.px.len & LEN_PLUS) && (v1.val.px.len < (v2.val.px.len & LEN_MASK)))
-      return 0;
-    if ((v2.val.px.len & LEN_RANGE) && ((v1.val.px.len < (0xff & (v2.val.px.len >> 16)))
-					|| (v1.val.px.len > (0xff & (v2.val.px.len >> 8)))))
-      return 0;
-    return 1;    
+    int l, h;
+    f_prefix_get_bounds(&v2.val.px, &l, &h);
+
+    return ((l <= v1.val.px.len) && (v1.val.px.len <= h));
   }
   return CMP_ERROR;
 }
@@ -195,6 +218,9 @@ val_in_range(struct f_val v1, struct f_val v2)
   if (res != CMP_ERROR)
     return res;
   
+  if ((v1.type == T_PREFIX) && (v2.type == T_PREFIX_SET))
+    return trie_match_prefix(v2.val.ti, &v1.val.px);
+
   if (v2.type == T_SET)
     switch (v1.type) {
     case T_ENUM:
@@ -244,11 +270,12 @@ val_print(struct f_val v)
   case T_IP: PRINTF( "%I", v.val.px.ip ); break;
   case T_PREFIX: PRINTF( "%I/%d", v.val.px.ip, v.val.px.len ); break;
   case T_PAIR: PRINTF( "(%d,%d)", v.val.i >> 16, v.val.i & 0xffff ); break;
+  case T_PREFIX_SET: trie_print(v.val.ti, buf, 2040); break;
   case T_SET: tree_print( v.val.t ); PRINTF( "\n" ); break;
   case T_ENUM: PRINTF( "(enum %x)%d", v.type, v.val.i ); break;
   case T_PATH: as_path_format(v.val.ad, buf2, 1020); PRINTF( "(path %s)", buf2 ); break;
   case T_CLIST: int_set_format(v.val.ad, 1, buf2, 1020); PRINTF( "(clist %s)", buf2 ); break;
-  case T_PATH_MASK: pm_format(v.val.path_mask, buf2, 1020); PRINTF( "(pathmask %s)", buf2 ); break;
+  case T_PATH_MASK: pm_format(v.val.path_mask, buf2, 1020); PRINTF( "(pathmask%s)", buf2 ); break;
   default: PRINTF( "[unknown type %x]", v.type );
 #undef PRINTF
   }
@@ -277,8 +304,10 @@ rta_cow(void)
   }
 }
 
+static struct rate_limit rl_runtime_err;
+
 #define runtime(x) do { \
-    log( L_ERR "filters, line %d: %s", what->lineno, x); \
+    log_rl(&rl_runtime_err, L_ERR "filters, line %d: %s", what->lineno, x); \
     res.type = T_RETURN; \
     res.val.i = F_ERROR; \
     return res; \
@@ -424,25 +453,38 @@ interpret(struct f_inst *what)
     switch (res.type = v2.type) {
     case T_VOID: runtime( "Can't assign void values" );
     case T_ENUM:
-    case T_INT: 
-    case T_IP: 
-    case T_PREFIX: 
-    case T_PAIR: 
+    case T_BOOL:
+    case T_INT:
+    case T_PAIR:
+    case T_STRING:
+    case T_IP:
+    case T_PREFIX:
+    case T_PREFIX_SET:
+    case T_SET:
     case T_PATH:
-    case T_CLIST:
     case T_PATH_MASK:
+    case T_CLIST:
       if (sym->class != (SYM_VARIABLE | v2.type))
 	runtime( "Assigning to variable of incompatible type" );
-      * (struct f_val *) sym->aux2 = v2; 
+      * (struct f_val *) sym->def = v2; 
       break;
     default:
       bug( "Set to invalid type" );
     }
     break;
 
-  case 'c':	/* integer (or simple type) constant */
+    /* some constants have value in a2, some in *a1.p, strange. */
+  case 'c':	/* integer (or simple type) constant, string, set, or prefix_set */
     res.type = what->aux;
-    res.val.i = what->a2.i;
+
+    if (res.type == T_PREFIX_SET)
+      res.val.ti = what->a2.p;
+    else if (res.type == T_SET)
+      res.val.t = what->a2.p;
+    else if (res.type == T_STRING)
+      res.val.s = what->a2.p;
+    else
+      res.val.i = what->a2.i;
     break;
   case 'C':
     res = * ((struct f_val *) what->a1.p);
@@ -497,6 +539,9 @@ interpret(struct f_inst *what)
       case T_ENUM:
 	res.val.i = * ((char *) rta + what->a2.i);
 	break;
+      case T_STRING:	/* Warning: this is a special case for proto attribute */
+	res.val.s = rta->proto->name;
+	break;
       case T_PREFIX:	/* Warning: this works only for prefix of network */
 	{
 	  res.val.px.ip = (*f_rte)->net->n.prefix;
@@ -546,6 +591,15 @@ interpret(struct f_inst *what)
 	res.type = T_INT;
 	res.val.i = e->u.data;
 	break;
+      case EAF_TYPE_IP_ADDRESS:
+	if (!e) {
+	  res.type = T_VOID;
+	  break;
+	}
+	res.type = T_IP;
+	struct adata * ad = e->u.ptr;
+	res.val.px.ip = * (ip_addr *) ad->data;
+	break;
       case EAF_TYPE_AS_PATH:
 	if (!e) {
 	  res.type = T_VOID;
@@ -585,6 +639,14 @@ interpret(struct f_inst *what)
 	  runtime( "Setting int attribute to non-int value" );
 	l->attrs[0].u.data = v1.val.i;
 	break;
+      case EAF_TYPE_IP_ADDRESS:
+	if (v1.type != T_IP)
+	  runtime( "Setting ip attribute to non-ip value" );
+	int len = sizeof(ip_addr);
+	struct adata *ad = lp_alloc(f_pool, sizeof(struct adata) + len);
+	ad->length = len;
+	(* (ip_addr *) ad->data) = v1.val.px.ip;
+	break;
       case EAF_TYPE_AS_PATH:
 	if (v1.type != T_PATH)
 	  runtime( "Setting path attribute to non-path value" );
@@ -630,7 +692,7 @@ interpret(struct f_inst *what)
     switch(v1.type) {
     case T_PREFIX: res.val.i = v1.val.px.len; break;
     case T_PATH:   res.val.i = as_path_getlen(v1.val.ad); break;
-    default: bug( "Length of what?" );
+    default: runtime( "Prefix or path expected" );
     }
     break;
   case P('c','p'):	/* Convert prefix to ... */
@@ -648,7 +710,7 @@ interpret(struct f_inst *what)
     ONEARG;
     res = v1;
     res.type |= T_RETURN;
-    break;
+    return res;
   case P('c','a'): /* CALL: this is special: if T_RETURN and returning some value, mask it out  */
     ONEARG;
     res = interpret(what->a2.p);
@@ -669,7 +731,10 @@ interpret(struct f_inst *what)
 	}
       }	
       /* It is actually possible to have t->data NULL */
-      return interpret(t->data);
+
+      res = interpret(t->data);
+      if (res.type & T_RETURN)
+	return res;
     }
     break;
   case P('i','M'): /* IP.MASK(val) */
@@ -782,16 +847,21 @@ i_same(struct f_inst *f1, struct f_inst *f2)
     break;
 
   case 'c': 
-    if (f1->aux & T_SET) {
+    switch (f1->aux) {
+
+    case T_PREFIX_SET:
+      if (!trie_same(f1->a2.p, f2->a2.p))
+	return 0;
+
+    case T_SET:
       if (!same_tree(f1->a2.p, f2->a2.p))
 	return 0;
-      break;
-    } 
-    switch (f1->aux) {
+
     case T_STRING:
       if (strcmp(f1->a2.p, f2->a2.p))
 	return 0;
       break;
+
     default:
       A2_SAME;
     }
