@@ -578,7 +578,10 @@ iface_patts_equal(list *a, list *b, int (*comp)(struct iface_patt *, struct ifac
   y = HEAD(*b);
   while (x->n.next && y->n.next)
     {
-      if (strcmp(x->pattern, y->pattern) ||
+      if ((!x->pattern && y->pattern) ||	/* This nasty lines where written by me... :-( Feela */
+          (!y->pattern && x->pattern) ||
+          (!(x->pattern==y->pattern) &&
+          strcmp(x->pattern, y->pattern)) ||
 	  !ipa_equal(x->prefix, y->prefix) ||
 	  x->pxlen != y->pxlen ||
 	  comp && !comp(x, y))
@@ -609,7 +612,7 @@ if_show_addr(struct ifa *a)
     opp[0] = 0;
   cli_msg(-1003, "\t%I/%d (%s%s%s, scope %s%s)",
 	  a->ip, a->pxlen,
-	  (a->flags & IA_PRIMARY) ? "Primary" : (a->flags & IA_SECONDARY) ? "Secondary" : "???",
+	  (a->flags & IA_PRIMARY) ? "Primary" : (a->flags & IA_SECONDARY) ? "Secondary" : "Unselected",
 	  broad, opp,
 	  ip_scope_text(a->scope),
 	  (a->flags & IA_UNNUMBERED) ? ", unnumbered" : "");
