@@ -76,16 +76,6 @@ bgp_check_as_path(struct bgp_proto *p, byte *a, int len)
   return bgp_check_path(a, len, p->as4_session ? 4 : 2, 11);
 }
 
-static int
-bgp_check_as4_path(struct bgp_proto *p, byte *a, int len)
-{
-  if (bgp_as4_support && (! p->as4_session))
-    return bgp_check_path(a, len, 4, 9);
-  else 
-    return 0;
-}
-
-
 static int
 bgp_check_next_hop(struct bgp_proto *p UNUSED, byte *a, int len)
 {
@@ -111,6 +101,27 @@ bgp_check_aggregator(struct bgp_proto *p, byte *a UNUSED, int len)
   return (len == exp_len) ? 0 : 5;
 }
 
+static void
+bgp_format_aggregator(eattr *a, byte *buf, int buflen UNUSED)
+{
+  struct adata *ad =  a->u.ptr;
+  byte *data = ad->data;
+  u32 as;
+
+  if (bgp_as4_support)
+    {
+      as = get_u32(data);
+      data += 4;
+    }
+  else
+    {
+      as = get_u16(data);
+      data += 2;
+    }
+
+  bsprintf(buf, "%d.%d.%d.%d AS%d", data[0], data[1], data[2], data[3], as);
+}
+
 static int
 bgp_check_cluster_list(struct bgp_proto *p UNUSED, byte *a UNUSED, int len)
 {
@@ -159,7 +170,7 @@ static struct attr_desc bgp_attr_table[] = {
   { "atomic_aggr", 0, BAF_TRANSITIVE, EAF_TYPE_OPAQUE, 1,			/* BA_ATOMIC_AGGR */
     NULL, NULL },
   { "aggregator", -1, BAF_OPTIONAL | BAF_TRANSITIVE, EAF_TYPE_OPAQUE, 1,	/* BA_AGGREGATOR */
-    bgp_check_aggregator, NULL },
+    bgp_check_aggregator, bgp_format_aggregator },
   { "community", -1, BAF_OPTIONAL | BAF_TRANSITIVE, EAF_TYPE_INT_SET, 1,	/* BA_COMMUNITY */
     NULL, NULL },
   { "originator_id", 4, BAF_OPTIONAL, EAF_TYPE_ROUTER_ID, 0,			/* BA_ORIGINATOR_ID */
@@ -175,8 +186,8 @@ static struct attr_desc bgp_attr_table[] = {
     bgp_check_unreach_nlri, NULL },
   { NULL, },									/* BA_EXTENDED_COMM */
   { "as4_path", -1, BAF_OPTIONAL | BAF_TRANSITIVE, EAF_TYPE_OPAQUE, 1,		/* BA_AS4_PATH */
-    bgp_check_as4_path, NULL },
-  { "as4_aggregator", 8, BAF_OPTIONAL | BAF_TRANSITIVE, EAF_TYPE_OPAQUE, 1,	/* BA_AS4_PATH */
+    NULL, NULL },
+  { "as4_aggregator", -1, BAF_OPTIONAL | BAF_TRANSITIVE, EAF_TYPE_OPAQUE, 1,	/* BA_AS4_PATH */
     NULL, NULL }
 };
 
@@ -326,7 +337,7 @@ bgp_get_attr_len(eattr *a)
  * The bgp_encode_attrs() function takes a list of extended attributes
  * and converts it to its BGP representation (a part of an Update message).
  *
- * Result: Length of the attribute block generated.
+ * Result: Length of the attribute block generated or -1 if not enough space.
  */
 unsigned int
 bgp_encode_attrs(struct bgp_proto *p, byte *w, ea_list *attrs, int remains)
@@ -365,6 +376,7 @@ bgp_encode_attrs(struct bgp_proto *p, byte *w, ea_list *attrs, int remains)
 	  int new_used;
 	  int nl = as_path_convert_to_old(a->u.ptr, buf, &new_used);
 
+	  DBG("BGP: Encoding old AS_PATH\n");
 	  rv = bgp_encode_attr_hdr(w, BAF_TRANSITIVE, BA_AS_PATH, nl);
 	  ADVANCE(w, remains, rv);
 	  memcpy(w, buf, nl);
@@ -381,6 +393,7 @@ bgp_encode_attrs(struct bgp_proto *p, byte *w, ea_list *attrs, int remains)
 	   * discarded in bgp_check_as_path().
 	   */
 
+	  DBG("BGP: Encoding AS4_PATH\n");
 	  rv = bgp_encode_attr_hdr(w, BAF_OPTIONAL | BAF_TRANSITIVE, BA_AS4_PATH, len);
 	  ADVANCE(w, remains, rv);
 	  memcpy(w, a->u.ptr->data, len);
@@ -465,8 +478,7 @@ bgp_encode_attrs(struct bgp_proto *p, byte *w, ea_list *attrs, int remains)
   return w - start;
 
  err_no_buffer:
-  log(L_ERR "BGP: attribute list too long, ignoring the remaining attributes");
-  return w - start;
+  return -1;
 }
 
 static void
@@ -1041,6 +1053,85 @@ bgp_merge_as_paths(struct adata *old2, struct adata *old4, int req_as, struct li
   return newa;
 }
 
+static int
+as4_aggregator_valid(struct adata *aggr)
+{
+  if (aggr->length != 8)
+    return 0;
+
+  u32 *a = (u32 *) aggr->data;
+
+  if ((a[0] == 0) || (a[1] == 0))
+    return 0;
+}
+
+static int
+as4_path_sanitize_and_get_length(struct adata *path)
+{
+  int res = 0;
+  u8 *p, *dst;
+  int len, plen, copy;
+
+  dst = p = path->data;
+  len = path->length;
+
+  while (len)
+    {
+      if (len <= 2) /* We do not allow empty segments */
+	goto inconsistent_path;
+
+      switch (p[0])
+	{
+	case AS_PATH_SET:
+	  plen = 2 + 4 * p[1];
+	  copy = 1;
+	  res++;
+	  break;
+
+	case AS_PATH_SEQUENCE:
+	  plen = 2 + 4 * p[1];
+	  copy = 1;
+	  res += p[1];
+	  break;
+
+	case AS_PATH_CONFED_SEQUENCE:
+	case AS_PATH_CONFED_SET:
+	  log(L_WARN "BGP: AS4_PATH attribute contains AS_CONFED_* segment, skipping segment");
+	  plen = 2 + 4 * p[1];
+	  copy = 0;
+	  break;
+
+	default:
+	  goto unknown_segment;
+	}
+
+      if (len < plen)
+	goto inconsistent_path;
+
+      if (copy)
+	{
+	  if (dst != p)
+	    memmove(dst, p, plen);
+	  dst += plen;
+	}
+
+      len -= plen;
+      p += plen;
+    }
+
+  path->length = dst - path->data;
+  return res;
+
+ inconsistent_path:
+  log(L_WARN "BGP: AS4_PATH attribute is inconsistent, skipping attribute");
+  return -1;
+
+ unknown_segment:
+  log(L_WARN "BGP: AS4_PATH attribute contains unknown segment, skipping attribute");
+  return -1;
+}
+
+
 
 /* Reconstruct 4B AS_PATH and AGGREGATOR according to RFC 4893 4.2.3 */
 static void
@@ -1050,6 +1141,14 @@ bgp_reconstruct_4b_atts(struct bgp_proto *p, rta *a, struct linpool *pool)
   eattr *p4 =ea_find(a->eattrs, EA_CODE(EAP_BGP, BA_AS4_PATH));
   eattr *a2 =ea_find(a->eattrs, EA_CODE(EAP_BGP, BA_AGGREGATOR));
   eattr *a4 =ea_find(a->eattrs, EA_CODE(EAP_BGP, BA_AS4_AGGREGATOR));
+  int a4_removed = 0;
+
+  if (a4 && !as4_aggregator_valid(a4->u.ptr))
+    {
+      log(L_WARN "BGP: AS4_AGGREGATOR attribute is invalid, skipping attribute");
+      a4 = NULL;
+      a4_removed = 1;
+    }
 
   if (a2)
     {
@@ -1081,7 +1180,7 @@ bgp_reconstruct_4b_atts(struct bgp_proto *p, rta *a, struct linpool *pool)
 	  /* Common case, use old AGGREGATOR attribute */
 	  a2->u.ptr = bgp_aggregator_convert_to_new(a2->u.ptr, pool);
 
-	  if (a2_as == AS_TRANS)
+	  if ((a2_as == AS_TRANS) && !a4_removed)
 	    log(L_WARN "BGP: AGGREGATOR attribute contain AS_TRANS, but AS4_AGGREGATOR is missing");
 	}
     }
@@ -1090,13 +1189,12 @@ bgp_reconstruct_4b_atts(struct bgp_proto *p, rta *a, struct linpool *pool)
       log(L_WARN "BGP: AS4_AGGREGATOR attribute received, but AGGREGATOR attribute is missing");
 
   int p2_len = as_path_getlen(p2->u.ptr);
-  int p4_len = p4 ? as_path_getlen(p4->u.ptr) : AS_PATH_MAXLEN;
+  int p4_len = p4 ? as4_path_sanitize_and_get_length(p4->u.ptr) : -1;
 
-  if (p2_len < p4_len)
+  if ((p4_len <= 0) || (p2_len < p4_len))
     p2->u.ptr = bgp_merge_as_paths(p2->u.ptr, NULL, AS_PATH_MAXLEN, pool);
   else
     p2->u.ptr = bgp_merge_as_paths(p2->u.ptr, p4->u.ptr, p2_len - p4_len, pool);
-
 }
 
 static void
@@ -1257,24 +1355,26 @@ bgp_decode_attrs(struct bgp_conn *conn, byte *attr, unsigned int len, struct lin
     }
 
 #ifdef IPV6
-  if (seen[BA_MP_REACH_NLRI / 8] & (1 << (BA_MP_REACH_NLRI % 8)))
+  /* If we received MP_REACH_NLRI we should check mandatory attributes */
+  if (bgp->mp_reach_len != 0)
     mandatory = 1;
 #endif
 
+  /* If there is no (reachability) NLRI, we should exit now */
+  if (! mandatory)
+    return a;
+
   /* Check if all mandatory attributes are present */
-  if (mandatory)
+  for(i=0; i < ARRAY_SIZE(bgp_mandatory_attrs); i++)
     {
-      for(i=0; i < ARRAY_SIZE(bgp_mandatory_attrs); i++)
+      code = bgp_mandatory_attrs[i];
+      if (!(seen[code/8] & (1 << (code%8))))
 	{
-	  code = bgp_mandatory_attrs[i];
-	  if (!(seen[code/8] & (1 << (code%8))))
-	    {
-	      bgp_error(conn, 3, 3, &bgp_mandatory_attrs[i], 1);
-	      return NULL;
-	    }
+	  bgp_error(conn, 3, 3, &bgp_mandatory_attrs[i], 1);
+	  return NULL;
 	}
     }
-  
+
   /* When receiving attributes from non-AS4-aware BGP speaker,
    * we have to reconstruct 4B AS_PATH and AGGREGATOR attributes
    */
@@ -1296,6 +1396,7 @@ bgp_decode_attrs(struct bgp_conn *conn, byte *attr, unsigned int len, struct lin
   /* If there's no local preference, define one */
   if (!(seen[0] & (1 << BA_LOCAL_PREF)))
     bgp_attach_attr(&a->eattrs, pool, BA_LOCAL_PREF, 0);
+
   return a;
 
 loop:
