@@ -17,8 +17,12 @@
 #include "lib/unaligned.h"
 #include "lib/socket.h"
 
+#include "nest/cli.h"
+
 #include "bgp.h"
 
+static struct rate_limit rl_rcv_update,  rl_snd_update;
+
 static byte *
 bgp_create_notification(struct bgp_conn *conn, byte *buf)
 {
@@ -43,6 +47,20 @@ bgp_put_cap_ipv6(struct bgp_conn *conn UNUSED, byte *buf)
   *buf++ = 1;		/* and SAFI 1 */
   return buf;
 }
+
+#else
+
+static byte *
+bgp_put_cap_ipv4(struct bgp_conn *conn UNUSED, byte *buf)
+{
+  *buf++ = 1;		/* Capability 1: Multiprotocol extensions */
+  *buf++ = 4;		/* Capability data length */
+  *buf++ = 0;		/* We support AF IPv4 */
+  *buf++ = BGP_AF_IPV4;
+  *buf++ = 0;		/* RFU */
+  *buf++ = 1;		/* and SAFI 1 */
+  return buf;
+}
 #endif
 
 static byte *
@@ -67,13 +85,27 @@ bgp_create_open(struct bgp_conn *conn, byte *buf)
   put_u16(buf+1, (p->local_as < 0xFFFF) ? p->local_as : AS_TRANS);
   put_u16(buf+3, p->cf->hold_time);
   put_u32(buf+5, p->local_id);
+
+  if (conn->start_state == BSS_CONNECT_NOCAP)
+    {
+      BGP_TRACE(D_PACKETS, "Skipping capabilities");
+      buf[9] = 0;
+      return buf + 10;
+    }
+
   /* Skipped 3 B for length field and Capabilities parameter header */
   cap = buf + 12;
 
+#ifndef IPV6
+  if (p->cf->advertise_ipv4)
+    cap = bgp_put_cap_ipv4(conn, cap);
+#endif
+
 #ifdef IPV6
   cap = bgp_put_cap_ipv6(conn, cap);
 #endif
-  if (p->cf->enable_as4)
+
+  if (conn->want_as4_support)
     cap = bgp_put_cap_as4(conn, cap);
 
   cap_len = cap - buf - 12;
@@ -115,6 +147,18 @@ bgp_encode_prefixes(struct bgp_proto *p, byte *w, struct bgp_bucket *buck, unsig
   return w - start;
 }
 
+static void
+bgp_flush_prefixes(struct bgp_proto *p, struct bgp_bucket *buck)
+{
+  while (!EMPTY_LIST(buck->prefixes))
+    {
+      struct bgp_prefix *px = SKIP_BACK(struct bgp_prefix, bucket_node, HEAD(buck->prefixes));
+      log(L_ERR "%s: - route %I/%d skipped", p->p.name, px->n.prefix, px->n.pxlen);
+      rem_node(&px->bucket_node);
+      fib_delete(&p->prefix_fib, px);
+    }
+}
+
 #ifndef IPV6		/* IPv4 version */
 
 static byte *
@@ -138,7 +182,7 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
     }
   put_u16(buf, wd_size);
 
-  if (remains >= 2048)
+  if (remains >= 3072)
     {
       while ((buck = (struct bgp_bucket *) HEAD(p->bucket_queue))->send_node.next)
 	{
@@ -149,8 +193,19 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
 	      bgp_free_bucket(p, buck);
 	      continue;
 	    }
+
 	  DBG("Processing bucket %p\n", buck);
-	  a_size = bgp_encode_attrs(p, w+2, buck->eattrs, 1024);
+	  a_size = bgp_encode_attrs(p, w+2, buck->eattrs, 2048);
+
+	  if (a_size < 0)
+	    {
+	      log(L_ERR "%s: Attribute list too long, skipping corresponding route group", p->p.name);
+	      bgp_flush_prefixes(p, buck);
+	      rem_node(&buck->send_node);
+	      bgp_free_bucket(p, buck);
+	      continue;
+	    }
+
 	  put_u16(w, a_size);
 	  w += a_size + 2;
 	  r_size = bgp_encode_prefixes(p, w, buck, remains - a_size);
@@ -165,7 +220,7 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
     }
   if (wd_size || r_size)
     {
-      BGP_TRACE(D_PACKETS, "Sending UPDATE");
+      BGP_TRACE_RL(&rl_snd_update, D_PACKETS, "Sending UPDATE");
       return w;
     }
   else
@@ -197,13 +252,14 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
       *tmp++ = 0;
       *tmp++ = BGP_AF_IPV6;
       *tmp++ = 1;
-      ea->attrs[0].u.ptr->length = bgp_encode_prefixes(p, tmp, buck, remains-11);
+      ea->attrs[0].u.ptr->length = 3 + bgp_encode_prefixes(p, tmp, buck, remains-11);
       size = bgp_encode_attrs(p, w, ea, remains);
+      ASSERT(size >= 0);
       w += size;
       remains -= size;
     }
 
-  if (remains >= 2048)
+  if (remains >= 3072)
     {
       while ((buck = (struct bgp_bucket *) HEAD(p->bucket_queue))->send_node.next)
 	{
@@ -214,8 +270,19 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
 	      bgp_free_bucket(p, buck);
 	      continue;
 	    }
+
 	  DBG("Processing bucket %p\n", buck);
-	  size = bgp_encode_attrs(p, w, buck->eattrs, 1024);
+	  size = bgp_encode_attrs(p, w, buck->eattrs, 2048);
+
+	  if (size < 0)
+	    {
+	      log(L_ERR "%s: Attribute list too long, ignoring corresponding route group", p->p.name);
+	      bgp_flush_prefixes(p, buck);
+	      rem_node(&buck->send_node);
+	      bgp_free_bucket(p, buck);
+	      continue;
+	    }
+
 	  w += size;
 	  remains -= size;
 	  tstart = tmp = bgp_attach_attr_wa(&ea, bgp_linpool, BA_MP_REACH_NLRI, remains-8);
@@ -262,7 +329,9 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
 	  *tmp++ = 0;			/* No SNPA information */
 	  tmp += bgp_encode_prefixes(p, tmp, buck, remains - (8+3+32+1));
 	  ea->attrs[0].u.ptr->length = tmp - tstart;
-	  w += bgp_encode_attrs(p, w, ea, remains);
+	  size = bgp_encode_attrs(p, w, ea, remains);
+	  ASSERT(size >= 0);
+	  w += size;
 	  break;
 	}
     }
@@ -272,7 +341,7 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
   lp_flush(bgp_linpool);
   if (size)
     {
-      BGP_TRACE(D_PACKETS, "Sending UPDATE");
+      BGP_TRACE_RL(&rl_snd_update, D_PACKETS, "Sending UPDATE");
       return w;
     }
   else
@@ -318,7 +387,8 @@ bgp_fire_tx(struct bgp_conn *conn)
 
   if (s & (1 << PKT_SCHEDULE_CLOSE))
     {
-      bgp_close_conn(conn);
+      /* We can finally close connection and enter idle state */
+      bgp_conn_enter_idle_state(conn);
       return 0;
     }
   if (s & (1 << PKT_NOTIFICATION))
@@ -371,8 +441,17 @@ bgp_schedule_packet(struct bgp_conn *conn, int type)
   DBG("BGP: Scheduling packet type %d\n", type);
   conn->packets_to_send |= 1 << type;
   if (conn->sk && conn->sk->tpos == conn->sk->tbuf)
-    while (bgp_fire_tx(conn))
-      ;
+    ev_schedule(conn->tx_ev);
+}
+
+void
+bgp_kick_tx(void *vconn)
+{
+  struct bgp_conn *conn = vconn;
+
+  DBG("BGP: kicking TX\n");
+  while (bgp_fire_tx(conn))
+    ;
 }
 
 void
@@ -406,9 +485,8 @@ bgp_parse_capabilities(struct bgp_conn *conn, byte *opt, int len)
 	case 65:
 	  if (cl != 4)
 	    goto err;
-	  p->as4_support = 1;
-	  p->as4_session = p->cf->enable_as4;
-	  if (p->as4_session)
+	  conn->peer_as4_support = 1;
+	  if (conn->want_as4_support)
 	    conn->advertised_as = get_u32(opt + 2);
 	  break;
 
@@ -427,6 +505,7 @@ bgp_parse_capabilities(struct bgp_conn *conn, byte *opt, int len)
 static int
 bgp_parse_options(struct bgp_conn *conn, byte *opt, int len)
 {
+  struct bgp_proto *p = conn->bgp;
   int ol;
 
   while (len > 0)
@@ -447,7 +526,10 @@ bgp_parse_options(struct bgp_conn *conn, byte *opt, int len)
       switch (opt[0])
 	{
 	case 2:
-	  bgp_parse_capabilities(conn, opt + 2, ol);
+	  if (conn->start_state == BSS_CONNECT_NOCAP)
+	    BGP_TRACE(D_PACKETS, "Ignoring received capabilities");
+	  else
+	    bgp_parse_capabilities(conn, opt + 2, ol);
 	  break;
 
 	default:
@@ -473,23 +555,23 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, int len)
   struct bgp_proto *p = conn->bgp;
   struct bgp_config *cf = p->cf;
   unsigned hold;
+  u16 base_as;
   u32 id;
 
   /* Check state */
   if (conn->state != BS_OPENSENT)
-    { bgp_error(conn, 5, 0, NULL, 0); }
+    { bgp_error(conn, 5, 0, NULL, 0); return; }
 
   /* Check message contents */
   if (len < 29 || len != 29 + pkt[28])
     { bgp_error(conn, 1, 2, pkt+16, 2); return; }
   if (pkt[19] != BGP_VERSION)
     { bgp_error(conn, 2, 1, pkt+19, 1); return; } /* RFC 1771 says 16 bits, draft-09 tells to use 8 */
-  conn->advertised_as = get_u16(pkt+20);
+  conn->advertised_as = base_as = get_u16(pkt+20);
   hold = get_u16(pkt+22);
   id = get_u32(pkt+24);
   BGP_TRACE(D_PACKETS, "Got OPEN(as=%d,hold=%d,id=%08x)", conn->advertised_as, hold, id);
 
-  p->remote_id = id; // ???
   if (bgp_parse_options(conn, pkt+29, pkt[28]))
     return;
 
@@ -499,11 +581,11 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, int len)
   if (!id || id == 0xffffffff || id == p->local_id)
     { bgp_error(conn, 2, 3, pkt+24, -4); return; }
 
+  if ((conn->advertised_as != base_as) && (base_as != AS_TRANS))
+    log(L_WARN "%s: Peer advertised inconsistent AS numbers", p->p.name);
 
   if (conn->advertised_as != p->remote_as)
-    {
-      bgp_error(conn, 2, 2, (byte *) &(conn->advertised_as), -4); return;
-    }
+    { bgp_error(conn, 2, 2, (byte *) &(conn->advertised_as), -4); return; }
 
   /* Check the other connection */
   other = (conn == &p->outgoing_conn) ? &p->incoming_conn : &p->outgoing_conn;
@@ -513,6 +595,7 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, int len)
     case BS_CONNECT:
     case BS_ACTIVE:
     case BS_OPENSENT:
+    case BS_CLOSE:
       break;
     case BS_OPENCONFIRM:
       if ((p->local_id < id) == (conn == &p->incoming_conn))
@@ -532,19 +615,13 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, int len)
       bug("bgp_rx_open: Unknown state");
     }
 
-  /* Make this connection primary */
-  conn->primary = 1;
-  p->conn = conn;
-
   /* Update our local variables */
-  if (hold < p->cf->hold_time)
-    conn->hold_time = hold;
-  else
-    conn->hold_time = p->cf->hold_time;
+  conn->hold_time = MIN(hold, p->cf->hold_time);
   conn->keepalive_time = p->cf->keepalive_time ? : conn->hold_time / 3;
-  // p->remote_as = conn->advertised_as;
   p->remote_id = id;
-  DBG("BGP: Hold timer set to %d, keepalive to %d, AS to %d, ID to %x\n", conn->hold_time, conn->keepalive_time, p->remote_as, p->remote_id);
+  p->as4_session = conn->want_as4_support && conn->peer_as4_support;
+
+  DBG("BGP: Hold timer set to %d, keepalive to %d, AS to %d, ID to %x, AS4 session to %d\n", conn->hold_time, conn->keepalive_time, p->remote_as, p->remote_id, p->as4_session);
 
   bgp_schedule_packet(conn, PKT_KEEPALIVE);
   bgp_start_timer(conn->hold_timer, conn->hold_time);
@@ -700,19 +777,14 @@ bgp_do_rx_update(struct bgp_conn *conn,
       /* Create fake NEXT_HOP attribute */
       if (len < 1 || (*x != 16 && *x != 32) || len < *x + 2)
 	goto bad;
-      memcpy(bgp_attach_attr_wa(&a0->eattrs, bgp_linpool, BA_NEXT_HOP, 16), x+1, 16);
-      len -= *x + 2;
-      x += *x + 1;
 
-      /* Ignore SNPA info */
-      i = *x++;
-      while (i--)
-	{
-	  if (len < 1 || len < 1 + *x)
-	    goto bad;
-	  len -= *x + 1;
-	  x += *x + 1;
-	}
+      byte *nh = bgp_attach_attr_wa(&a0->eattrs, bgp_linpool, BA_NEXT_HOP, 16);
+      memcpy(nh, x+1, 16);
+      ipa_ntoh(*(ip_addr *)nh);
+
+      /* Also ignore one reserved byte */
+      len -= *x + 2;
+      x += *x + 2;
 
       if (bgp_get_nexthop(p, a0))
 	{
@@ -750,7 +822,8 @@ bgp_rx_update(struct bgp_conn *conn, byte *pkt, int len)
   byte *withdrawn, *attrs, *nlri;
   int withdrawn_len, attr_len, nlri_len;
 
-  BGP_TRACE(D_PACKETS, "Got UPDATE");
+  BGP_TRACE_RL(&rl_rcv_update, D_PACKETS, "Got UPDATE");
+
   if (conn->state != BS_ESTABLISHED)
     { bgp_error(conn, 5, 0, NULL, 0); return; }
   bgp_start_timer(conn->hold_timer, conn->hold_time);
@@ -814,27 +887,52 @@ static struct {
   { 3, 11, "Malformed AS_PATH" },
   { 4, 0, "Hold timer expired" },
   { 5, 0, "Finite state machine error" },
-  { 6, 0, "Cease" }
+  { 6, 0, "Cease" }, /* Subcodes are according to [RFC4486] */
+  { 6, 1, "Maximum number of prefixes reached" },
+  { 6, 2, "Administrative shutdown" },
+  { 6, 3, "Peer de-configured" },
+  { 6, 4, "Administrative reset" },
+  { 6, 5, "Connection rejected" },
+  { 6, 6, "Other configuration change" },
+  { 6, 7, "Connection collision resolution" },
+  { 6, 8, "Out of Resources" }
 };
 
+/**
+ * bgp_error_dsc - return BGP error description
+ * @buff: temporary buffer
+ * @code: BGP error code
+ * @subcode: BGP error subcode
+ *
+ * bgp_error_dsc() returns error description for BGP errors
+ * which might be static string or given temporary buffer.
+ */
+const byte *
+bgp_error_dsc(byte *buff, unsigned code, unsigned subcode)
+{
+  unsigned i;
+  for (i=0; i < ARRAY_SIZE(bgp_msg_table); i++)
+    if (bgp_msg_table[i].major == code && bgp_msg_table[i].minor == subcode)
+      {
+	return bgp_msg_table[i].msg;
+      }
+
+  bsprintf(buff, "Unknown error %d.%d", code, subcode);
+  return buff;
+}
+
 void
 bgp_log_error(struct bgp_proto *p, char *msg, unsigned code, unsigned subcode, byte *data, unsigned len)
 {
-  byte *name, namebuf[16];
+  const byte *name;
+  byte namebuf[32];
   byte *t, argbuf[36];
   unsigned i;
 
   if (code == 6 && !subcode)		/* Don't report Cease messages */
     return;
 
-  bsprintf(namebuf, "%d.%d", code, subcode);
-  name = namebuf;
-  for (i=0; i < ARRAY_SIZE(bgp_msg_table); i++)
-    if (bgp_msg_table[i].major == code && bgp_msg_table[i].minor == subcode)
-      {
-	name = bgp_msg_table[i].msg;
-	break;
-      }
+  name = bgp_error_dsc(namebuf, code, subcode);
   t = argbuf;
   if (len)
     {
@@ -852,15 +950,42 @@ bgp_log_error(struct bgp_proto *p, char *msg, unsigned code, unsigned subcode, b
 static void
 bgp_rx_notification(struct bgp_conn *conn, byte *pkt, int len)
 {
+  struct bgp_proto *p = conn->bgp;
   if (len < 21)
     {
       bgp_error(conn, 1, 2, pkt+16, 2);
       return;
     }
-  bgp_log_error(conn->bgp, "Received error notification", pkt[19], pkt[20], pkt+21, len-21);
-  conn->error_flag = 1;
-  if (conn->primary)
-    proto_notify_state(&conn->bgp->p, PS_STOP);
+
+  unsigned code = pkt[19];
+  unsigned subcode = pkt[20];
+  int delay = 1;
+
+  bgp_log_error(conn->bgp, "Received error notification", code, subcode, pkt+21, len-21);
+  bgp_store_error(conn->bgp, conn, BE_BGP_RX, (code << 16) | subcode);
+
+#ifndef IPV6
+  if ((code == 2) && ((subcode == 4) || (subcode == 7))
+      /* Error related to capability:
+       * 4 - Peer does not support capabilities at all.
+       * 7 - Peer request some capability. Strange unless it is IPv6 only peer.
+       */
+      && (p->cf->capabilities == 2)
+      /* Capabilities are not explicitly enabled or disabled, therefore heuristic is used */
+      && (conn->start_state == BSS_CONNECT)
+      /* Failed connection attempt have used capabilities */
+      && (p->cf->remote_as <= 0xFFFF))
+      /* Not possible with disabled capabilities */
+    {
+      /* We try connect without capabilities */
+      log(L_WARN "%s: Capability related error received, retry with capabilities disabled", p->p.name);
+      conn->bgp->start_state = BSS_CONNECT_NOCAP;
+      delay = 0;
+    }
+#endif
+
+  if (delay) bgp_update_startup_delay(conn->bgp, conn, code, subcode);
+  bgp_conn_enter_close_state(conn);
   bgp_schedule_packet(conn, PKT_SCHEDULE_CLOSE);
 }
 
@@ -874,10 +999,7 @@ bgp_rx_keepalive(struct bgp_conn *conn)
   switch (conn->state)
     {
     case BS_OPENCONFIRM:
-      DBG("BGP: UP!!!\n");
-      conn->state = BS_ESTABLISHED;
-      bgp_attr_init(conn->bgp);
-      proto_notify_state(&conn->bgp->p, PS_UP);
+      bgp_conn_enter_established_state(conn);
       break;
     case BS_ESTABLISHED:
       break;
@@ -930,18 +1052,8 @@ bgp_rx(sock *sk, int size)
   DBG("BGP: RX hook: Got %d bytes\n", size);
   while (end >= pkt_start + BGP_HEADER_LENGTH)
     {
-      if (conn->error_flag)
-	{
-	  /*
-	   *  We still need to remember the erroneous packet, so that
-	   *  we can generate error notifications properly.  To avoid
-	   *  subsequent reads rewriting the buffer, we just reset the
-	   *  rx_hook.
-	   */
-	  DBG("BGP: Error, dropping input\n");
-	  sk->rx_hook = NULL;
-	  return 0;
-	}
+      if ((conn->state == BS_CLOSE) || (conn->sk != sk))
+	return 0;
       for(i=0; i<16; i++)
 	if (pkt_start[i] != 0xff)
 	  {
