@@ -1,7 +1,7 @@
 /*
  *      BIRD -- OSPF
  *
- *      (c) 1999 - 2004 Ondrej Filip <feela@network.cz>
+ *      (c) 1999--2005 Ondrej Filip <feela@network.cz>
  *
  *      Can be freely distributed and used under the terms of the GNU GPL.
  *
@@ -17,5 +17,9 @@ void ospf_iface_notify(struct proto *p, unsigned flags, struct iface *iface);
 void ospf_iface_info(struct ospf_iface *ifa);
 void ospf_iface_shutdown(struct ospf_iface *ifa);
 void ospf_iface_new(struct proto_ospf *po, struct iface *iface, struct ospf_area_config *ac, struct ospf_iface_patt *ip);
+void ospf_iface_change_mtu(struct proto_ospf *po, struct ospf_iface *ifa);
+void ospf_set_rxbuf_size(struct ospf_iface *ifa, u32 rxbuf);
+
+
 
 #endif /* _BIRD_OSPF_IFACE_H_ */
