@@ -75,13 +75,25 @@ ospf_pkt_finalize(struct ospf_iface *ifa, struct ospf_packet *pkt)
 
       pkt->checksum = 0;
 
+      /* Perhaps use random value to prevent replay attacks after
+	 reboot when system does not have independent RTC? */
       if (!ifa->csn)
-        ifa->csn = (u32) time(NULL);
+	{
+	  ifa->csn = (u32) now;
+	  ifa->csn_use = now;
+	}
+
+      /* We must have sufficient delay between sending a packet and increasing 
+	 CSN to prevent reordering of packets (in a network) with different CSNs */
+      if ((now - ifa->csn_use) > 1)
+	ifa->csn++;
+
+      ifa->csn_use = now;
 
       pkt->u.md5.keyid = passwd->id;
       pkt->u.md5.len = OSPF_AUTH_CRYPT_SIZE;
       pkt->u.md5.zero = 0;
-      pkt->u.md5.csn = htonl(ifa->csn++);
+      pkt->u.md5.csn = htonl(ifa->csn);
       tail = ((void *)pkt) + ntohs(pkt->length);
       MD5Init(&ctxt);
       MD5Update(&ctxt, (char *) pkt, ntohs(pkt->length));
@@ -126,7 +138,7 @@ ospf_pkt_checkauth(struct ospf_neighbor *n, struct ospf_iface *ifa, struct ospf_
       break;
     case OSPF_AUTH_SIMPLE:
       pass = password_find(ifa->passwords, 1);
-      if(!pass)
+      if (!pass)
       {
         OSPF_TRACE(D_PACKETS, "OSPF_auth: no password found");
 	return 0;
@@ -149,6 +161,7 @@ ospf_pkt_checkauth(struct ospf_neighbor *n, struct ospf_iface *ifa, struct ospf_
         OSPF_TRACE(D_PACKETS, "OSPF_auth: wrong size of md5 digest");
         return 0;
       }
+
       if (ntohs(pkt->length) + OSPF_AUTH_CRYPT_SIZE != size)
       {
         OSPF_TRACE(D_PACKETS, "OSPF_auth: size mismatch (%d vs %d)",
@@ -164,28 +177,33 @@ ospf_pkt_checkauth(struct ospf_neighbor *n, struct ospf_iface *ifa, struct ospf_
 
       tail = ((void *)pkt) + ntohs(pkt->length);
 
-      WALK_LIST(ptmp, *(ifa->passwords))
+      if (ifa->passwords)
       {
-        if (pkt->u.md5.keyid != ptmp->id) continue;
-        if ((ptmp->accfrom > now_real) || (ptmp->accto < now_real)) continue;
-        pass = ptmp;
-        break;
+	WALK_LIST(ptmp, *(ifa->passwords))
+	{
+	  if (pkt->u.md5.keyid != ptmp->id) continue;
+	  if ((ptmp->accfrom > now_real) || (ptmp->accto < now_real)) continue;
+	  pass = ptmp;
+	  break;
+	}
       }
 
-      if(!pass)
+      if (!pass)
       {
         OSPF_TRACE(D_PACKETS, "OSPF_auth: no suitable md5 password found");
         return 0;
       }
 
-      if(n)
+      if (n)
       {
-        if(ntohs(pkt->u.md5.csn) < n->csn)
-        {
-          OSPF_TRACE(D_PACKETS, "OSPF_auth: lower sequence number");
-          return 0;
-        }
-        n->csn = ntohs(pkt->u.md5.csn);
+	u32 rcv_csn = ntohl(pkt->u.md5.csn);
+	if(rcv_csn < n->csn)
+	{
+	  OSPF_TRACE(D_PACKETS, "OSPF_auth: lower sequence number (rcv %d, old %d)", rcv_csn, n->csn);
+	  return 0;
+	}
+
+	n->csn = rcv_csn;
       }
 
       MD5Init(&ctxt);
@@ -232,6 +250,12 @@ ospf_rx_hook(sock * sk, int size)
 
   ps = (struct ospf_packet *) ipv4_skip_header(sk->rbuf, &size);
 
+  if (ps == NULL)
+  {
+    log(L_ERR "%s%I - bad IP header", mesg, sk->faddr);
+    return 1;
+  }
+
   if ((ifa->oa->areaid != 0) && (ntohl(ps->areaid) == 0))
   {
     WALK_LIST(iff, po->iface_list)
@@ -247,11 +271,6 @@ ospf_rx_hook(sock * sk, int size)
   DBG("%s: RX_Hook called on interface %s.\n", p->name, sk->iface->name);
 
   osize = ntohs(ps->length);
-  if (ps == NULL)
-  {
-    log(L_ERR "%s%I - bad IP header", mesg, sk->faddr);
-    return 1;
-  }
 
   if ((unsigned) size < sizeof(struct ospf_packet))
   {
@@ -397,6 +416,9 @@ ospf_send_to(sock *sk, ip_addr ip, struct ospf_iface *ifa)
   int len = ntohs(pkt->length) + ((ifa->autype == OSPF_AUTH_CRYPT) ? OSPF_AUTH_CRYPT_SIZE : 0);
   ospf_pkt_finalize(ifa, pkt);
 
+  if (sk->tbuf != sk->tpos)
+    log(L_ERR "Aiee, old packet was overwritted in TX buffer");
+
   if (ipa_equal(ip, IPA_NONE))
     sk_send(sk, len);
   else
