@@ -309,6 +309,7 @@ originate_net_lsa(struct ospf_iface *ifa)
 	       ifa->iface->name);
     ifa->nlsa->lsa.sn += 1;
     ifa->nlsa->lsa.age = LSA_MAXAGE;
+    lsasum_calculate(&ifa->nlsa->lsa, ifa->nlsa->lsa_body);
     ospf_lsupd_flood(NULL, NULL, &ifa->nlsa->lsa, NULL, ifa->oa, 0);
     s_rem_node(SNODE ifa->nlsa);
     if (ifa->nlsa->lsa_body != NULL)
@@ -538,9 +539,9 @@ check_sum_lsa(struct proto_ospf *po, ort *nf, int dest)
   if ((nf->n.type > RTS_OSPF_IA) && (nf->o.type > RTS_OSPF_IA)) return;
 
 #ifdef LOCAL_DEBUG
-  DBG("Checking...dest = %d, %I/%d", dest, nf->fn.prefix, nf->fn.pxlen);
-  if (nf->n.oa) DBG("New: met=%d, oa=%d", nf->n.metric1, nf->n.oa->areaid);
-  if (nf->o.oa) DBG("Old: met=%d, oa=%d", nf->o.metric1, nf->o.oa->areaid);
+  DBG("Checking...dest = %d, %I/%d\n", dest, nf->fn.prefix, nf->fn.pxlen);
+  if (nf->n.oa) DBG("New: met=%d, oa=%d\n", nf->n.metric1, nf->n.oa->areaid);
+  if (nf->o.oa) DBG("Old: met=%d, oa=%d\n", nf->o.metric1, nf->o.oa->areaid);
 #endif
 
   WALK_LIST(oa, po->area_list)
@@ -907,7 +908,7 @@ ospf_dump_lsa(struct top_hash_entry *he, struct proto *p)
     case LSA_T_NET:
       ln = he->lsa_body;
       rts = (u32 *) (ln + 1);
-      max = (he->lsa.length - sizeof(struct ospf_lsa_header) -                                                                                                               
+      max = (he->lsa.length - sizeof(struct ospf_lsa_header) -
 		sizeof(struct ospf_lsa_net)) / sizeof(u32);
 
       for (i = 0; i < max; i++)
