@@ -12,6 +12,8 @@
 
 #define CONFIG_UNIX_IFACE
 #define CONFIG_UNIX_SET
+#define CONFIG_UNIX_DONTROUTE
+#undef CONFIG_SKIP_MC_BIND
 #define CONFIG_LINUX_SCAN
 
 #define CONFIG_LINUX_MC_MREQN
