@@ -19,9 +19,9 @@ struct krt_set_params {
 struct krt_set_status {
 };
 
-static inline void krt_set_construct(struct krt_config *c) { };
-static inline void krt_set_start(struct krt_proto *p, int first) { };
-static inline void krt_set_shutdown(struct krt_proto *p, int last) { };
-static inline int krt_set_params_same(struct krt_set_params *o, struct krt_set_params *n) { return 1; }
+static inline void krt_set_construct(struct krt_config *c UNUSED) { };
+static inline void krt_set_start(struct krt_proto *p UNUSED, int first UNUSED) { };
+static inline void krt_set_shutdown(struct krt_proto *p UNUSED, int last UNUSED) { };
+static inline int krt_set_params_same(struct krt_set_params *o UNUSED, struct krt_set_params *n UNUSED) { return 1; }
 
 #endif
