@@ -15,6 +15,6 @@ struct krt_set_params {
 struct krt_set_status {
 };
 
-static inline int krt_set_params_same(struct krt_set_params *o, struct krt_set_params *n) { return 1; }
+static inline int krt_set_params_same(struct krt_set_params *o UNUSED, struct krt_set_params *n UNUSED) { return 1; }
 
 #endif
