@@ -91,13 +91,16 @@ static int
 unix_read_config(struct config **cp, char *name)
 {
   struct config *conf = config_alloc(name);
+  int ret;
 
   *cp = conf;
   conf_fd = open(name, O_RDONLY);
   if (conf_fd < 0)
     return 0;
   cf_read_hook = cf_read;
-  return config_parse(conf);
+  ret = config_parse(conf);
+  close(conf_fd);
+  return ret;
 }
 
 static void
@@ -435,6 +438,11 @@ main(int argc, char **argv)
       if (pid)
 	return 0;
       setsid();
+      close(0);
+      if (open("/dev/null", O_RDWR) < 0)
+	die("Cannot open /dev/null: %m");
+      dup2(0, 1);
+      dup2(0, 2);
     }
 
   signal_init();
