@@ -9,7 +9,7 @@
 #ifndef _BIRD_TIMER_H_
 #define _BIRD_TIMER_H_
 
-#include <sys/time.h>
+#include <time.h>
 
 #include "lib/resource.h"
 
@@ -30,9 +30,11 @@ void tm_start(timer *, unsigned after);
 void tm_stop(timer *);
 void tm_dump_all(void);
 
-extern bird_clock_t now;		/* Time in seconds since unknown epoch */
+extern bird_clock_t now; 		/* Relative, monotonic time in seconds */
+extern bird_clock_t now_real;		/* Time in seconds since fixed known epoch */
 
 bird_clock_t tm_parse_date(char *);	/* Convert date to bird_clock_t */
+bird_clock_t tm_parse_datetime(char *);	/* Convert date to bird_clock_t */
 void tm_format_date(char *, bird_clock_t);	/* Convert bird_clock_t to date */
 #define TM_DATE_BUFFER_SIZE 12		/* Buffer size required by tm_format_date */
 void tm_format_datetime(char *, bird_clock_t);	/* Convert bird_clock_t to date + time */
